@@ -2,6 +2,7 @@ from __future__ import absolute_import
 
 import ssl
 import pytest
+from mock import Mock
 
 from urllib3.connectionpool import (
     connection_from_url,
@@ -31,6 +32,7 @@ from socket import error as SocketError
 from ssl import SSLError as BaseSSLError
 
 from dummyserver.server import DEFAULT_CA
+from test import SHORT_TIMEOUT
 
 
 class HTTPUnixConnection(HTTPConnection):
@@ -212,13 +214,13 @@ class TestConnectionPool(object):
 
     def test_max_connections(self):
         with HTTPConnectionPool(host="localhost", maxsize=1, block=True) as pool:
-            pool._get_conn(timeout=0.01)
+            pool._get_conn(timeout=SHORT_TIMEOUT)
 
             with pytest.raises(EmptyPoolError):
-                pool._get_conn(timeout=0.01)
+                pool._get_conn(timeout=SHORT_TIMEOUT)
 
             with pytest.raises(EmptyPoolError):
-                pool.request("GET", "/", pool_timeout=0.01)
+                pool.request("GET", "/", pool_timeout=SHORT_TIMEOUT)
 
             assert pool.num_connections == 1
 
@@ -278,19 +280,27 @@ class TestConnectionPool(object):
 
             # Make sure that all of the exceptions return the connection
             # to the pool
-            _test(Empty, EmptyPoolError)
             _test(BaseSSLError, MaxRetryError, SSLError)
             _test(CertificateError, MaxRetryError, SSLError)
 
             # The pool should never be empty, and with these two exceptions
             # being raised, a retry will be triggered, but that retry will
             # fail, eventually raising MaxRetryError, not EmptyPoolError
-            # See: https://github.com/shazow/urllib3/issues/76
+            # See: https://github.com/urllib3/urllib3/issues/76
             pool._make_request = lambda *args, **kwargs: _raise(HTTPException)
             with pytest.raises(MaxRetryError):
-                pool.request("GET", "/", retries=1, pool_timeout=0.01)
+                pool.request("GET", "/", retries=1, pool_timeout=SHORT_TIMEOUT)
             assert pool.pool.qsize() == POOL_SIZE
 
+    def test_empty_does_not_put_conn(self):
+        """Do not put None back in the pool if the pool was empty"""
+
+        with HTTPConnectionPool(host="localhost", maxsize=1, block=True) as pool:
+            pool._get_conn = Mock(side_effect=EmptyPoolError(pool, "Pool is empty"))
+            pool._put_conn = Mock(side_effect=AssertionError("Unexpected _put_conn"))
+            with pytest.raises(EmptyPoolError):
+                pool.request("GET", "/")
+
     def test_assert_same_host(self):
         with connection_from_url("http://google.com:80") as c:
             with pytest.raises(HostChangedError):
@@ -348,9 +358,9 @@ class TestConnectionPool(object):
             assert pool.timeout._connect == Timeout.DEFAULT_TIMEOUT
             assert pool.timeout.total is None
 
-            pool = HTTPConnectionPool(host="localhost", timeout=3)
-            assert pool.timeout._read == 3
-            assert pool.timeout._connect == 3
+            pool = HTTPConnectionPool(host="localhost", timeout=SHORT_TIMEOUT)
+            assert pool.timeout._read == SHORT_TIMEOUT
+            assert pool.timeout._connect == SHORT_TIMEOUT
             assert pool.timeout.total is None
 
     def test_no_host(self):
@@ -425,7 +435,7 @@ class TestConnectionPool(object):
         would be released if the initial request failed, even if a retry
         succeeded.
 
-        [1] <https://github.com/shazow/urllib3/issues/651>
+        [1] <https://github.com/urllib3/urllib3/issues/651>
         """
 
         class _raise_once_make_request_function(object):
@@ -450,7 +460,6 @@ class TestConnectionPool(object):
 
         def _test(exception):
             with HTTPConnectionPool(host="localhost", maxsize=1, block=True) as pool:
-
                 # Verify that the request succeeds after two attempts, and that the
                 # connection is left on the response object, instead of being
                 # released back into the pool.
