@@ -10,7 +10,8 @@ from urllib3.exceptions import (
     ProtocolError,
 )
 from urllib3.response import httplib
-from urllib3.util.ssl_ import HAS_SNI
+from urllib3 import util
+from urllib3.util import ssl_wrap_socket
 from urllib3.util import ssl_
 from urllib3.util.timeout import Timeout
 from urllib3.util.retry import Retry
@@ -20,9 +21,8 @@ from dummyserver.testcase import SocketDummyServerTestCase, consume_socket
 from dummyserver.server import (
     DEFAULT_CERTS,
     DEFAULT_CA,
-    COMBINED_CERT_AND_KEY,
-    PASSWORD_KEYFILE,
     get_unreachable_address,
+    encrypt_key_pem,
 )
 
 from .. import onlyPy3, LogRecorder
@@ -36,20 +36,26 @@ except ImportError:
 
 
 from collections import OrderedDict
+import os.path
 from threading import Event
+import os
 import select
 import socket
+import shutil
 import ssl
+import tempfile
 import mock
 
 import pytest
+import trustme
 
 from test import (
-    fails_on_travis_gce,
     requires_ssl_context_keyfile_password,
     SHORT_TIMEOUT,
     LONG_TIMEOUT,
     notPyPy2,
+    notSecureTransport,
+    resolvesLocalhostFQDN,
 )
 
 # Retry failed tests
@@ -81,8 +87,9 @@ class TestCookies(SocketDummyServerTestCase):
 
 
 class TestSNI(SocketDummyServerTestCase):
-    @pytest.mark.skipif(not HAS_SNI, reason="SNI-support not available")
     def test_hostname_in_first_request_packet(self):
+        if not util.HAS_SNI:
+            pytest.skip("SNI-support not available")
         done_receiving = Event()
         self.buf = b""
 
@@ -99,7 +106,8 @@ class TestSNI(SocketDummyServerTestCase):
                 pool.request("GET", "/", retries=0)
             except MaxRetryError:  # We are violating the protocol
                 pass
-            done_receiving.wait()
+            successful = done_receiving.wait(LONG_TIMEOUT)
+            assert successful, "Timed out waiting for connection accept"
             assert (
                 self.host.encode("ascii") in self.buf
             ), "missing hostname in SSL handshake"
@@ -110,6 +118,28 @@ class TestClientCerts(SocketDummyServerTestCase):
     Tests for client certificate support.
     """
 
+    @classmethod
+    def setup_class(cls):
+        cls.tmpdir = tempfile.mkdtemp()
+        ca = trustme.CA()
+        cert = ca.issue_cert(u"localhost")
+        encrypted_key = encrypt_key_pem(cert.private_key_pem, b"letmein")
+
+        cls.ca_path = os.path.join(cls.tmpdir, "ca.pem")
+        cls.cert_combined_path = os.path.join(cls.tmpdir, "server.combined.pem")
+        cls.cert_path = os.path.join(cls.tmpdir, "server.pem")
+        cls.key_path = os.path.join(cls.tmpdir, "key.pem")
+        cls.password_key_path = os.path.join(cls.tmpdir, "password_key.pem")
+
+        ca.cert_pem.write_to_path(cls.ca_path)
+        cert.private_key_and_cert_chain_pem.write_to_path(cls.cert_combined_path)
+        cert.cert_chain_pems[0].write_to_path(cls.cert_path)
+        cert.private_key_pem.write_to_path(cls.key_path)
+        encrypted_key.write_to_path(cls.password_key_path)
+
+    def teardown_class(cls):
+        shutil.rmtree(cls.tmpdir)
+
     def _wrap_in_ssl(self, sock):
         """
         Given a single socket, wraps it in TLS.
@@ -118,9 +148,9 @@ class TestClientCerts(SocketDummyServerTestCase):
             sock,
             ssl_version=ssl.PROTOCOL_SSLv23,
             cert_reqs=ssl.CERT_REQUIRED,
-            ca_certs=DEFAULT_CA,
-            certfile=DEFAULT_CERTS["certfile"],
-            keyfile=DEFAULT_CERTS["keyfile"],
+            ca_certs=self.ca_path,
+            certfile=self.cert_path,
+            keyfile=self.key_path,
             server_side=True,
         )
 
@@ -158,10 +188,10 @@ class TestClientCerts(SocketDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host,
             self.port,
-            cert_file=DEFAULT_CERTS["certfile"],
-            key_file=DEFAULT_CERTS["keyfile"],
+            cert_file=self.cert_path,
+            key_file=self.key_path,
             cert_reqs="REQUIRED",
-            ca_certs=DEFAULT_CA,
+            ca_certs=self.ca_path,
         ) as pool:
             pool.request("GET", "/", retries=0)
             done_receiving.set()
@@ -202,9 +232,9 @@ class TestClientCerts(SocketDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host,
             self.port,
-            cert_file=COMBINED_CERT_AND_KEY,
+            cert_file=self.cert_combined_path,
             cert_reqs="REQUIRED",
-            ca_certs=DEFAULT_CA,
+            ca_certs=self.ca_path,
         ) as pool:
             pool.request("GET", "/", retries=0)
             done_receiving.set()
@@ -230,7 +260,7 @@ class TestClientCerts(SocketDummyServerTestCase):
 
         self._start_server(socket_handler)
         with HTTPSConnectionPool(
-            self.host, self.port, cert_reqs="REQUIRED", ca_certs=DEFAULT_CA
+            self.host, self.port, cert_reqs="REQUIRED", ca_certs=self.ca_path
         ) as pool:
             with pytest.raises(MaxRetryError):
                 pool.request("GET", "/", retries=0)
@@ -277,9 +307,7 @@ class TestClientCerts(SocketDummyServerTestCase):
         self._start_server(socket_handler)
         ssl_context = ssl_.SSLContext(ssl_.PROTOCOL_SSLv23)
         ssl_context.load_cert_chain(
-            certfile=DEFAULT_CERTS["certfile"],
-            keyfile=PASSWORD_KEYFILE,
-            password=password,
+            certfile=self.cert_path, keyfile=self.password_key_path, password=password
         )
 
         with HTTPSConnectionPool(
@@ -287,7 +315,7 @@ class TestClientCerts(SocketDummyServerTestCase):
             self.port,
             ssl_context=ssl_context,
             cert_reqs="REQUIRED",
-            ca_certs=DEFAULT_CA,
+            ca_certs=self.ca_path,
         ) as pool:
             pool.request("GET", "/", retries=0)
             done_receiving.set()
@@ -308,8 +336,8 @@ class TestClientCerts(SocketDummyServerTestCase):
 
         with pytest.raises(expected_error):
             context.load_cert_chain(
-                certfile=DEFAULT_CERTS["certfile"],
-                keyfile=PASSWORD_KEYFILE,
+                certfile=self.cert_path,
+                keyfile=self.password_key_path,
                 password=b"letmei",
             )
 
@@ -469,7 +497,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
 
         try:
             self._start_server(socket_handler)
-            t = Timeout(connect=SHORT_TIMEOUT, read=LONG_TIMEOUT)
+            t = Timeout(connect=LONG_TIMEOUT, read=LONG_TIMEOUT)
             with HTTPConnectionPool(self.host, self.port, timeout=t) as pool:
                 response = pool.request("GET", "/", retries=1)
                 assert response.status == 200
@@ -506,7 +534,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
                 "/",
                 retries=0,
                 preload_content=False,
-                timeout=Timeout(connect=1, read=SHORT_TIMEOUT),
+                timeout=Timeout(connect=1, read=LONG_TIMEOUT),
             )
             try:
                 with pytest.raises(ReadTimeoutError):
@@ -1142,6 +1170,7 @@ class TestSSL(SocketDummyServerTestCase):
                 pool.request("GET", "/", retries=0)
             assert isinstance(cm.value.reason, SSLError)
 
+    @notSecureTransport
     def test_ssl_read_timeout(self):
         timed_out = Event()
 
@@ -1364,6 +1393,13 @@ class TestSSL(SocketDummyServerTestCase):
                         pool.request("GET", "/", timeout=SHORT_TIMEOUT)
                     context.load_default_certs.assert_not_called()
 
+    def test_load_verify_locations_exception(self):
+        """
+        Ensure that load_verify_locations raises SSLError for all backends
+        """
+        with pytest.raises(SSLError):
+            ssl_wrap_socket(None, ca_certs="/tmp/fake-file")
+
 
 class TestErrorWrapping(SocketDummyServerTestCase):
     def test_bad_statusline(self):
@@ -1474,7 +1510,7 @@ class TestHeaders(SocketDummyServerTestCase):
             pool.request("GET", "/", headers=OrderedDict(expected_request_headers))
             assert expected_request_headers == actual_request_headers
 
-    @fails_on_travis_gce
+    @resolvesLocalhostFQDN
     def test_request_host_header_ignores_fqdn_dot(self):
 
         received_headers = []
